package ch.jp.robwar;

/**
 * This class build on the BrainInterface to provide
 * an interface that gets rid of the 'steps' limitation.
 *
 * @author JP Martin
 * @date   3.13.1999
 **/
public class AsynchronousBrainInterface extends BrainInterface implements java.lang.Runnable {
  /* in the parent
  protected RobotBrain brain;
  protected int timeTilShoot;
  protected int timeTilFireRocket;
  */
  
  /* noop. This is important for both run and nexMove */
  protected Instructions robInstructions=new Instructions();
  protected Feedback robInput;
  
  /** 
   * The external robot and the simulation use this semaphore
   * so that reality cannot be altered in the middle of a step
   **/ 
  protected Object worldInterfaceLock = new Object(); 
  
  protected Thread myThread;

  /**
   * Nothing special about the constructor. However
   * remember that this class is not used the same way as
   * its parents (namely, you have to start() this one)
  **/
  public AsynchronousBrainInterface(RobotBrain newBrain) {
    super(newBrain);
  }
  
  
  /**
   * This method, meant to be called by
   * (new AsynchronousBrainInterface(myBrain)).run(),
   * Continuously calls the brain's nextMove method
   * and accumulates the results (think logical OR).
   *                                                      <p>
   * When BrainInterface.nextMove is called, it will
   * return this aggregated Instructions, and be given
   * new Stimuli. This stimuli will be passed to the
   * next nextMove and then blanked (think AND 0).
   * the time stimuli will remain the same, allowing
   * the RobotBrain to detect when new Stimuli are
   * fed to it.
   *                                                      <p>
   * The first thing this method does is suspend: it waits
   * for nextMove() to be called for the first time with some
   * input before it can start the robot and relay this world 
   * view to it. 
   *                                                      <p>
   * I will provide a nice method to stop the BrainInterface
   * someday when I see a use for it. Until then,
   * use stop().
   *                                                      <p>
   * This approach may not be perfect, and I may 
   * change it someday, especially if/when I decide   
   * to get rid of the notion of "step" altogether.
  **/
  public void run() {
  
    myThread=Thread.currentThread();
    try {
      myThread.setPriority(Thread.MIN_PRIORITY);
    } catch (SecurityException ouch) {
      ouch.printStackTrace();
      System.out.println("Error: Unable to lower thread priority. The program will run much slower than normal.");
    }
    
    try {
      myThread.suspend();
    } catch (SecurityException s) {
      s.printStackTrace();
      System.out.println("Fatal: error while waiting on run. Unable to run this robot.");
      return;
    }
    
    while (true) {
      
//      synchronized (worldInterfaceLock) {
      
        
        try {
          myThread.suspend();
        } catch (SecurityException s) {
          s.printStackTrace();
          System.out.println("Fatal: error while waiting on run. Unable to run this robot.");
          return;
        }
        /**/
  //    }
      
      
      robInstructions.mix(super.asynchronousNextMove(robInput));
      
      /* the simulator will call nextmove at some point, and
         this is when the instructions will be cleared and the 
         robInput filled with new weltanschau */         
         
    }
  } 
  
  /**
   * This will return the current robot's desire and update
   * its perception of the world. This represents a step.
  **/
  public Instructions nextMove(Feedback senses) {
    Instructions robotWill;
    
    try {
      myThread.resume();
    } catch (SecurityException se) {
      se.printStackTrace();
      System.out.println("Fatal: unable to wake up the robot thread.");
      System.out.println("This robot will be non-functional.");
    } catch (NullPointerException ne) {
      ne.printStackTrace();
      System.out.println("Error: nextMove called before start().");
      System.out.println("Please inform the author!");
    }
    
    //synchronized (worldInterfaceLock) {
      /* tell our parent that we enter a new time step */
      super.timeTick();
      /* grab a copy of the current instructions */
      /* (a copy to avoid having two threads share this object) */
      robotWill=new Instructions(robInstructions);
      /* update the robot's sensory inputs */
      robInput=senses;
      /* reset the instructions (for the next move) */
      robInstructions=new Instructions();
    //}
    
    /*
    System.out.println("Step start");
    try {
      Thread.currentThread().sleep(10);
    } catch (InterruptedException e) {
      System.out.println("Unable to wait for brain; resuming");      
    }
    System.out.println("Step stop");
    */
    
    return robotWill;
  }
  
  
}