package ch.jp.robwar;


/**
 * This class interspedes between the Simulation
 * and the RobotBrain, making sure that the Instructions
 * given by the RobotBrain are valid (for example, the
 * speed is within the limits of the robot).    
 *
 * @author JP Martin
 * @date   9.3.1998
 **/
public class BrainInterface {
  protected RobotBrain brain;
  protected int timeTilShoot;
  protected int timeTilFireRocket;
  
  public BrainInterface(RobotBrain newBrain) {
    brain=newBrain;
    timeTilShoot=0;
    timeTilFireRocket=0;
  }
  
  public RobotBrain getBrain() {
    return brain;
  }
  
  /** 
   * This goes to illustrate that OO is not perfect
   * (and/or -more likely- that I am not perfect ;)
   * This method had to be added to ensure good behavior
   * to both BrainInterface and the new asynchronousBrainInterface
  **/
  protected void timeTick() {
    if (timeTilShoot>0) timeTilShoot--;
    if (timeTilFireRocket>0) timeTilFireRocket--;
  }
  
  protected Instructions asynchronousNextMove(Feedback senses) {
    
    
    Instructions desire;
    
    desire=brain.nextMove(senses);

    if (desire!=null) {
      // tolerate sloppy rotation (but not too much)
      double rotation=desire.getRotation();
      if (rotation>=360) rotation-=360;
      if (rotation<-360) rotation+=360;
      // trim excessive rotation
      if (rotation>8) rotation=8;
      if (rotation<-8) rotation=-8;
      desire.setRotation(rotation);
      // trim excessive speed
      if (desire.getSpeed()>0.1) desire.setSpeed(0.1);
      if (desire.getSpeed()<-0.06) desire.setSpeed(-0.06);
      // delays between shots
      if (desire.getFire()) {
        if (timeTilShoot>0) {
          desire.setFire(false);
        } else {
          timeTilShoot=10;
        }
      }
      if (desire.getFireRocket()) {
        // timing is ok?
        if (timeTilFireRocket>0) {
          desire.setFireRocket(false);
        } else 
        // enough rockets?
        if (senses.selfInfo.nbRockets>0) {
          timeTilFireRocket=10;
          //senses.selfInfo.nbRockets--;
        } else {
          desire.setFireRocket(false);
        }
      }
    }
    return desire;   
  }
  
  public Instructions nextMove(Feedback senses) {
    
    timeTick();
    return asynchronousNextMove(senses);
    
  }
  
}